@@ -35,7 +35,7 @@
 #include "translate.h"
 #include "wave.h"
 
-const char *version_string = "1.45.23  03.May.11";
+const char *version_string = "1.45.24  06.May.11";
 const int version_phdata  = 0x014500;
 
 int option_device_number = -1;
@@ -739,7 +739,6 @@ static bool InterpretCondition(Translator *tr, int control, PHONEME_LIST *plist,
 
 			case 12:  // isVoiced
 				return((ph->type == phVOWEL) || (ph->type == phLIQUID) || (ph->phflags & phVOICED));
-			}
 
 			case 13:  // isFirstVowel
 				count = 0;
@@ -752,6 +751,28 @@ static bool InterpretCondition(Translator *tr, int control, PHONEME_LIST *plist,
 					plist--;
 				}
 				return(count==1);
+
+			case 14:  // isSeqFlag1
+				// is this preceded  by a sequence if 1 or more vowels which have 'flag1' ?  (lang=hi)
+				if(plist->sourceix != 0)
+					return(false);   // this is the first phoneme in the word, so no.
+
+				count = 0;
+				for(;;)
+				{
+					plist--;
+					if(plist->ph->type == phVOWEL)
+					{
+						if(plist->ph->phflags & phFLAG1)
+							count++;
+						else
+							break;  // stop when we find a vowel without flag1
+					}
+					if(plist->sourceix != 0)
+						break;
+				}
+				return(count > 0);
+			}
 			break;
 
 		}
